{% extends 'base.html' %}
{% load icon_tags %}
{% load crispy_forms_tags %}
{% load static %}

{% block content %}

<div class="tab-content mt-2">
  <div id="analyses" class="tab-pane active">
    <div class="tab-content">
      <span class="spinner"></span>
      <div>Please wait...</div>
    </div>
  </div>
</div>

{% endblock content %}

{% block javascript %}

<script src="{% static 'js/analysis_app.bundle.js' %}"></script>
<script>
  const params = new URLSearchParams(window.location.search);
  topobank.analysis_app.createAnalysisResultsListApp('#analyses', '{{ csrf_token }}', {
    subjects: params.get('subjects')
  });
</script>

{% endblock javascript %}
