
{% extends 'base.html' %}
{% load icon_tags %}


{% block content %}

  {% load crispy_forms_tags %}

  <div class="tab-content mt-2">
    {% if form.errors %}
      {% for field in form %}
          {% for error in field.errors %}
              <div class="alert alert-danger">
                  <strong>Field {{ field.name }}: {{ error|escape }}</strong>
              </div>
          {% endfor %}
      {% endfor %}
      {% for error in form.non_field_errors %}
          <div class="alert alert-danger">
              <strong>{{ error|escape }}</strong>
          </div>
      {% endfor %}
    {% endif %}

    <div class="container">
      <div class="row">
        {% crispy form form.helper %}
      </div>
    </div>


  </div>

{% endblock content %}

