{% extends "base.html" %}
{% load static i18n %}


{% block content %}

  <div class="tab-content mt-2 extra-tab">
    <div class="tab-pane active">

      <div class="row">
        <div class="col-2">

          <div class="nav nav-pills nav-pills-custom flex-column" role="tablist" aria-orientation="vertical">
            <a class="nav-link mb-3 p-3 active" data-toggle="pill" href="#basic_usage" role="tab" aria-selected="true">
              Basic Usage
            </a>
            <a class="nav-link mb-3 p-3" data-toggle="pill" href="#file_formats" role="tab" aria-selected="false">
              File Formats
            </a>
          </div>
        </div>

        <div class="col-10">
          <div class="tab-content mt-4">
            <div class="tab-pane fade show active" id="basic_usage">
            {% include "pages/basic_usage.html" %}
            </div>
            <div class="tab-pane fade" id="file_formats">
            {% include "manager/file_formats.html" %}
            </div>

          </div>
        </div>
      </div>
    </div>
  </div>

{% endblock content %}

{% block javascript %}



{% endblock javascript %}
