{% extends "base.html" %}

{% block title %}Page not found{% endblock %}

{% block content %}
<h1>Page not found</h1>

<p>This is not the page you were looking for.
  Maybe the URL is no longer valid because some resources have been deleted.</p>
{% endblock content %}
