{% extends "base.html" %}
{% load static i18n %}
{% block title %}Members{% endblock %}

{% block content %}
<div class="container">
  <h2>Users</h2>

  <div class="list-group">
    {% for user in user_list %}
      <a href="{% url 'users:detail' user.username %}" class="list-group-item">
        <h4 class="list-group-item-heading">{{ user.username }}</h4>
      </a>
    {% endfor %}
  </div>
</div>
{% endblock content %}
