export const ccLicenseInfo = {
    'cc0-1.0': {
        descriptionUrl: 'https://creativecommons.org/publicdomain/zero/1.0/',
        legalCodeUrl: 'https://creativecommons.org/publicdomain/zero/1.0/legalcode',
        title: 'CC0 1.0 Universal',
        optionName: 'CC0 1.0 (Public Domain Dedication)',
        spdxIdentifier: 'CC0-1.0'
    },
    'ccby-4.0': {
        descriptionUrl: 'https://creativecommons.org/licenses/by/4.0/',
        legalCodeUrl: 'https://creativecommons.org/licenses/by/4.0/legalcode',
        title: 'Creative Commons Attribution 4.0 International Public License',
        optionName: 'CC BY 4.0',
        spdxIdentifier: 'CC-BY-4.0'
    },
    'ccbysa-4.0': {
        descriptionUrl: 'https://creativecommons.org/licenses/by-sa/4.0/',
        legalCodeUrl: 'https://creativecommons.org/licenses/by-sa/4.0/legalcode',
        title: 'Creative Commons Attribution-ShareAlike 4.0 International Public License',
        optionName: 'CC BY-SA 4.0',
        spdxIdentifier: 'CC-BY-SA-4.0'
    }
}
