{% extends 'base.html' %}
{% load icon_tags %}

{% block content %}

<div class="card">
 <h3 class="card-header">
   {% fa5_icon 'flash' %}
   Server error
 </h3>
 <div class="card-body">
   <div class="alert alert-error" role="alert">
     Oops, something went wrong! We're sorry.
     <p>We track these errors automatically, but if the problem persists please
       <a href="#" data-toggle="modal" data-target="#contactModal">contact us</a>.</p>
   </div>
    <a class="btn btn-primary" href="{% url 'home' %}">Take me back to the site</a>
 </div>
</div>

{% endblock content %}
