<h1>Supported File Formats</h1>

<p>Extending the supported file formats is a continuous work in progress.
If
<ul>
  <li>you have a question or problem regarding the upload of
      topography data, or</li>
  <li>your file is not supported, or</li>
  <li>you need more information on this page here,
  </li>
</ul>
please send us an <a href="mailto:{{ contact_email_address }}">email</a>
or open an <a href="https://github.com/ComputationalMechanics/TopoBank/issues" target="_blank">issue on GitHub</a>.
Thank you for your help.</li>
</p>

<p>
The following file formats are supported. <b>Please click on a file format name in order
  to expand the description for that format:</b>
</p>

<div id="file-formats">
  {% for name, format, description in reader_infos %}
  <div class="card">
    <div class="card-header" id="header-format-{{ format }}">
      <a class="collapsed card-link" data-toggle="collapse" href="#collapse-format-{{ format }}">
        {{ name }}
      </a>
      <span class="float-end">{{ format }}</span>
    </div>
    <div id="collapse-format-{{ format }}" class="collapse" aria-labelledby="header-format-{{ format }}" data-parent="#file-formats">
      <div class="card-body">
        {{ description|safe }}
      </div>
    </div>
  </div>
  {% endfor %}
</div> <!-- /container -->
