{% extends 'base.html' %}
{% load static %}

{% block content %}

    <div class="tab-content pt-2 extra-tab">
        <div id="app" class="tab-pane active">
        </div>
    </div>

{% endblock content %}

{% block javascript %}

    <script src="{% static 'js/topography_detail_app.bundle.js' %}"></script>
    <script>
        const params = new URLSearchParams(window.location.search);
        const topographyId = Number.parseInt(params.get('topography'));
        const topographyUrl = `${window.location.origin}/manager/api/topography/${topographyId}/`;
        topobank.topography_detail_app.createTopographyDetailApp('#app', '{{ csrf_token }}', {
            topographyUrl: topographyUrl
        });
    </script>

{% endblock javascript %}
