{% extends 'base.html' %}
{% load icon_tags %}

{% block content %}

<div class="card">
 <h3 class="card-header">
   {% fa5_icon 'flash' %}
   Access denied.
 </h3>
 <div class="card-body">
   <div class="alert alert-warning" role="alert">
     This URL doesn't reference your data or you have no longer
     access because some data is no longer shared with you.
     If you think you should have access, please
     <a href="#" data-toggle="modal" data-target="#contactModal">contact</a> the provider of this application.
   </div>
 </div>
</div>

{% endblock content %}
