{% extends "base.html" %}
{% load crispy_forms_tags %}
{% load account %}

{% block title %}{{ user.username }}{% endblock %}

{% block content %}
  <h1>{% user_display user %}</h1>
  <form class="form-horizontal" method="post" action="{% url 'users:update' %}">
    {% csrf_token %}
    {{ form|crispy }}
    <div class="control-group">
      <div class="controls">
        <button type="submit" class="btn">Update</button>
      </div>
    </div>
  </form>
{% endblock %}
