{% load icon_tags %}
<h1>Basic Usage</h1>

Here are a few workflows which should help you to get started with this web application.
If you are missing a description here, please
open an <a href="https://github.com/ComputationalMechanics/TopoBank/issues" target="_blank">issue on GitHub</a>.
Thank you.

<hr>

<h2>Uploading new data</h2>

<ol>
  <li>Visit the
    <a href="{% url 'ce_ui:select' %}">{% fa5_icon 'check-square' 'far' %} Datasets</a> tab.</li>
  <li>If you want to add a measurement to an existing surface, press its "View" button.
  Otherwise press the "Create surface" button and fill out the form, press "Save".</li>
  <li>The detail page for the surface opens. Press "Add measurement" and follow the dialogue.</li>
</ol>

<hr>

<h2>Analyzing surfaces and measurements</h2>


<h3>Comparing analysis results from multiple surfaces/measurements</h3>
<ol>
  <li>First visit <a href="{% url 'ce_ui:select' %}">{% fa5_icon 'check-square' 'far' %} Find&Select</a>,
  and use the checkboxes to select one or more surface and/or measurements.
    You can see the measurements by clicking on the small triangle
   on the left-hand side of a surface's name. The selected items appear in the bar underneath the
  top navigation bar, which always shows the <em>current selection</em>.</li>
  <li>Next to the current selection a button appears:
    <a class="btn btn-sm btn-outline-success" href="{% url 'ce_ui:results-list' %}">
      {% fa5_icon 'chart-area' %} Analyze
    </a>
    Press it in order to view analyses for the current selection.
  </li>
  <li>Another tab opens named <b>{% fa5_icon 'chart-area' %} Analyze</b>.
  <li>Select one or more Analysis function by clicking on their checkboxes, e.g. choose
  {% fa5_icon 'check-square' 'far' %} <em>Height Distribution</em>
  and {% fa5_icon 'check-square' 'far' %} <em>Power Spectrum</em>.</li>
  <li>Press the button <button class="btn btn-primary btn-sm">Update results</button>.</li>
  <li>For each analysis function a card is shown. You can use the toolbar at the plot to zoom/pan etc. or
  download the plot. Some analysis plots may provide additional feature's. They are accessible
  by pressing the "Open" button. Then a new tab opens showing only results for the related analysis function.</li>
</ol>


<h3>Showing analysis results for a single surface or measurement</h3>

If you want to analyze a single surface or measurement, there are also alternative ways:

<ol>
  <li>First visit the <a href="{% url 'ce_ui:select' %}">{% fa5_icon 'check-square' 'far' %} Find&Select</a> tab, and
  find the surface you are interested in. Press the "Analyze" button on the right-hand side of the row.
  Then automatically, this surface is selected with all of its measurements and
  the "Analyze" tab is loaded. This also works for measurements.</li>
  <li>If you want to analyze a single measurement, there is also another way.
  You can enter the surface by pressing the button "View".
  Now open the measurement, either by pressing the corresponding bar in the
  bandwidth plot or by sweeping through a gallery of measurements and clicking on one thumbnail.
  After the detail page for the measurement has opened, press "Analyze this measurement".
  The old selection is then replaced by this measurement, and the
    <b>{% fa5_icon 'chart-area' %} Analyze</b>
  tab opens.</li>
</ol>

<h3>Averaging analysis results over several measurements</h3>

Some analysis functions like <em>Power Spectrum</em> or <em>Autocorrelation</em> are not only
calculated over single measurements but also as an average over all measurements
of a surface. When selecting analysis function for display, these functions are denoted with <em>"(Average)"</em>
in their name. If a surface has more then one measurement, additionally a thick line with an average is
plotted on top of the other lines. The average is also included in the download of the data, together with
an standard error estimate of the mean.




