export function applyDefaultBokehStyle(figure) {
    figure.xaxis.axis_label_text_font_style = "normal";
    figure.yaxis.axis_label_text_font_style = "normal";
    figure.xaxis.major_label_text_font_size = "16px";
    figure.yaxis.major_label_text_font_size = "16px";
    figure.xaxis.axis_label_text_font_size = "16px";
    figure.yaxis.axis_label_text_font_size = "16px";
}
